## Code to generate simulated data for performance testing of PhySortR package.
#

##########################
#### Define Functions ####
##########################

## Generate random trees using template topologies.
## The template topologies (e.g. "t100_template.tre") are passed as the argument 'template.tree'
generateRandTrees <- function(path.to.trees, template.tree, no.key.leaves, no.normal.leaves, no.dummy.leaves, no.trees) {
  library(PhySortR)
  library(ape)
  
  # Generate the names of key leaves.
  key.node.storage <- character()
  for (x in 1:(no.key.leaves/2)) {
    key.node.storage <- c(key.node.storage, paste('TARGET_A_', x, sep=""))
    key.node.storage <- c(key.node.storage, paste('TARGET_B_', x, sep=""))
  }
  
  # Generate the names of normal leaves. 
  tip.storage <- character()
  for (x in 1:no.normal.leaves) {
    tip.storage <- c(tip.storage, paste('TIP_', x, sep=""))
  }
  
  # Generate the names of dummy leaves. To be deleted later to create random trees.
  for (x in 1:no.dummy.leaves) {
    tip.storage <- c(tip.storage, paste('X_', x, sep=""))
  }
  
  # Takes the leaf labels and randomly assigns them to a position in the tree. 
  raw <- file(template.tree)
  tree <- readLines(raw) # To excluse comments
  close(raw)
  for (x in 1:length(key.node.storage)) {
    rand.int <- sample(1:length(key.node.storage), 1)
    tree <- gsub(paste("\\$\\{key_node_", x, "\\}", sep=""), key.node.storage[rand.int], tree)
    # remove each elements as it is used.
    key.node.storage <- key.node.storage[-rand.int]
  }
  
  # Takes the normal and dummy leaf labels and randomly assigns them to a position in the tree. 
  for (x in 1:length(tip.storage)) {
    rand.int <- sample(1:length(tip.storage), 1)
    tree <- gsub(paste("\\$\\{tip_", x, "\\}", sep=""), tip.storage[rand.int], tree)
    # remove each elements as it is used.
    tip.storage <- tip.storage[-rand.int]
  }
  
  # Convert tree to standard Newick format. Read as class Phylo using APE.
  tree <- PhySortR::convert.eNewick(tree)
  tree.phylo <- ape::read.tree(text=tree)
  # Removes dummy leaves that have been randomly distributed throughout the tree.
  # Will result in a new tree with a novel topology. 
  for (x in 1:no.dummy.leaves) {
    tree.phylo <- ape::drop.tip(tree.phylo, paste("X_", x, sep=""))
  }
  # Convert from class Phylo to standard Newick string. 
  tree <- ape::write.tree(tree.phylo)
  
  #####################################
  #####################################
  # Convert standard Newick tree to extended Newick tree. 
  tree.split <- strsplit(tree, ")")
  tree.new <- list()
  # For each split 'node'.
  for (x in 1:length(tree.split[[1]])) {
    # Takes only the support value from eNewick ":x.xxx[<HERE>]..." 
    # If gsub() does not find a match, node.support == tree.split[[1]][x]
    node.support <- gsub("(^\\d*)\\:.*", "\\1", tree.split[[1]][x])
    # Check that node.support != tree.split[[1]][x] and thus has found the pattern.
    if (! tree.split[[1]][x] %in% node.support) {
      # Takes everythin but '<[node_support]>'.
      node.new1 <- gsub("(^\\d*)(\\:\\d*\\.\\d*)(.*)", "\\2", tree.split[[1]][x])
      node.new2 <- gsub("(^\\d*\\:\\d*\\.\\d*)(.*)", "\\2", tree.split[[1]][x])
      # Joins the 'node_support:node.new'.
      tree.new <- c(tree.new, paste(node.new1, "[", node.support, "]", node.new2, sep=""))
    } else { # If no node_support value found.
      tree.new <- c(tree.new, tree.split[[1]][x])
    }
  }
  # Paste converted parts back together.
  tree <- paste(tree.new, collapse=")")
  #####################################
  #####################################
  
  # Populate data set with copies of simulated tree
  for (x in 1:no.trees) {
    OUT <- file(paste(path.to.trees, "Tree_", x, ".tre", sep=""), 'w')
    write(tree, OUT)
    close(OUT)
  }
}

## Generate dataset replicates (reps) containing randomized trees
createDataSet <- function(reps, data.dir, template.tree, no.key.leaves, no.normal.leaves, no.dummy.leaves, no.trees) {
	dir.create(data.dir)
	for (x in 1:reps) {
	  rep.path = paste(data.dir, "/Rep_", as.character(x), "/", sep="")
	  dir.create(rep.path)
	  print(paste("  Rep", x))
	  generateRandTrees(rep.path, template.tree, no.key.leaves, no.normal.leaves, no.dummy.leaves, no.trees)
	}
}


##################
#### RUN CODE ####
##################
# Generate simulated phylogenetic trees.

# No reps per variable tested below
reps = 10
# Temp folder for simulated trees. 
path.to.trees = "SimulatedData/"
unlink(path.to.trees, recursive = TRUE)
dir.create(path.to.trees)


##########################################################
## Taxa-X                                               ##
##########################################################
data.dir = paste(path.to.trees, "Taxa-X", sep="/")
dir.create(data.dir)

# taxa=100 Trees=1000
print("#### taxa=100 Trees=1000")
createDataSet(reps, paste(data.dir, "/100", sep=""), "t100_template.tre", 20, 80, 5, 1000)

# taxa=200 Trees=1000
print("#### taxa=200 Trees=1000")
createDataSet(reps, paste(data.dir, "/200", sep=""), "t200_template.tre", 40, 160, 10, 1000)

# taxa=300 Trees=1000
print("#### taxa=300 Trees=1000")
createDataSet(reps, paste(data.dir, "/300", sep=""), "t300_template.tre", 60, 240, 15, 1000)

# taxa=400 Trees=1000
print("#### taxa=400 Trees=1000")
createDataSet(reps, paste(data.dir, "/400", sep=""), "t400_template.tre", 80, 320, 20, 1000)

# taxa=500 Trees=1000
print("#### taxa=500 Trees=1000")
createDataSet(reps, paste(data.dir, "/500", sep=""), "t500_template.tre", 100, 400, 25, 1000)


##########################################################
## Trees-N                                              ##
##########################################################
data.dir = paste(path.to.trees, "Trees-N", sep="/")
dir.create(data.dir)

# taxa=100 Trees=1000
print("#### taxa=100 Trees=1000")
createDataSet(reps, paste(data.dir, "/1000", sep=""), "t100_template.tre", 20, 80, 5, 1000)

# taxa=100 Trees=2000
print("#### taxa=100 Trees=2000")
createDataSet(reps, paste(data.dir, "/2000", sep=""), "t100_template.tre", 20, 80, 5, 2000)

# taxa=100 Trees=4000
print("#### taxa=100 Trees=4000")
createDataSet(reps, paste(data.dir, "/4000", sep=""), "t100_template.tre", 20, 80, 5, 4000)

# taxa=100 Trees=6000
print("#### taxa=100 Trees=6000")
createDataSet(reps, paste(data.dir, "/6000", sep=""), "t100_template.tre", 20, 80, 5, 6000)

# taxa=100 Trees=8000
print("#### taxa=100 Trees=8000")
createDataSet(reps, paste(data.dir, "/8000", sep=""), "t100_template.tre", 20, 80, 5, 8000)

# taxa=100 Trees=10000
print("#### taxa=100 Trees=10000")
createDataSet(reps, paste(data.dir, "/10000", sep=""), "t100_template.tre", 20, 80, 5, 10000)


##########################################################
## Negative-Taxa-X                                      ##
##########################################################
data.dir = paste(path.to.trees, "Negative-Taxa-X", sep="/")
dir.create(data.dir)

# Negative taxa=100 Trees=1000
print("#### Negative taxa=100 Trees=1000")
createDataSet(reps, paste(data.dir, "/100", sep=""), "t100_template_Negative.tre", 0, 100, 5, 1000)

# Negative taxa=200 Trees=1000
print("#### Negative taxa=200 Trees=1000")
createDataSet(reps, paste(data.dir, "/200", sep=""), "t200_template_Negative.tre", 0, 200, 10, 1000)

# Negative taxa=300 Trees=1000
print("#### Negative taxa=300 Trees=1000")
createDataSet(reps, paste(data.dir, "/300", sep=""), "t300_template_Negative.tre", 0, 300, 15, 1000)

# Negative taxa=400 Trees=1000
print("#### Negative taxa=400 Trees=1000")
createDataSet(reps, paste(data.dir, "/400", sep=""), "t400_template_Negative.tre", 0, 400, 20, 1000)

# Negative taxa=500 Trees=1000
print("#### Negative taxa=500 Trees=1000")
createDataSet(reps, paste(data.dir, "/500", sep=""), "t500_template_Negative.tre", 0, 500, 25, 1000)

##################
##################
##################